/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package block;

import game.PacMan;
import game.Sprite;
import main.SoundEffects;
import score.Score;

/**
 *
 * @author s070280
 */
public class Path extends Block{
    public Path(String typeName){
        changePath(typeName);
    }
    
    public Path(){
        
    }
    
    @Override
    public String[] getTypes(){
        String[] types = new String[PathType.values().length];
        int i=0;
        for(PathType p: PathType.values()){
            types[i] = this.getClass().getSimpleName().toLowerCase()+ "_" +p.toString().toLowerCase();
            i++;
        }
        return types;
    }
    
    private void changePath(String typeName){
        setImage(this.getClass().getSimpleName().toLowerCase() + "_" + typeName);
        pathType = PathType.valueOf(typeName.toUpperCase());                
    }   
    
    /**
     * Get the effect of the path on the passing sprite.
     * When the sprite is of type PacMan, change the path if necessary, update score and change PacMan's state
     * @param sprite: the sprite that passes this path
     * @param score: to update score when pacman eats a pacdot, fruit or powerpellet
     * @return boolean true: all sprite can pass a path
     */
    @Override
    public Boolean getEffect(Sprite sprite, Score score){
        // If the particular sprite passing by is of type pacman, set its State accoring to the path type.
        // In every other case, do nothing.
        if(sprite.getClass().getSimpleName().toLowerCase().equals("pacman") && !pathType.equals(PathType.EMPTY)){
            PacMan pacman = (PacMan) sprite; 
            
            if((pathType.equals(PathType.SWAMP) && pacman.pacState != pacman.pacState.SUPERMODE ) || pathType.equals(PathType.POWERPELLET) || pathType.equals(PathType.TURBO)){
                pacman.setState(pathType.nextState());
            }            
            score.addPoints(pathType.score());
            if (!pathType.equals(PathType.EMPTY)) {
                Thread tsound = new Thread(new SoundEffects(pathType.soundEffect));
                tsound.start();
            }
            if (!pathType.equals(PathType.SWAMP) && !pathType.equals(PathType.TURBO)) {
                changePath("empty");

            }       
            
        }        
        return true;
    }
    
    @Override
    public Boolean getEffect(Sprite sprite){     
        return true;
    }
    
    @Override
    public String getSaveTypeBlock() {
        return this.getClass().getSimpleName().toLowerCase() + "_" + pathType.toString().toLowerCase();
    }
    
    public enum PathType{
    EMPTY (0,"DEFAULT",""),
    PACDOT (100,"DEFAULT","dot"),
    POWERPELLET (500,"SUPERMODE","power"),
    FRUIT (3000,"DEFAULT","fruit"),
    TURBO (0,"TURBO","turbo"),
    SWAMP(0,"SWAMP","");
    
    private final int score;    
    private final String nextState;
    private final String soundEffect;
    
    PathType(int score, String nextState, String soundEffect){
        this.score = score;          
        this.nextState = nextState;
        this.soundEffect = soundEffect;
    }
    public int score() { return score; }
    public String nextState() { return nextState; }    
    public String soundEffect() { return soundEffect; }
    }
        
    private PathType pathType;
}

