/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package game;

import main.Global;
import java.lang.Math.*;
import java.util.logging.Logger;
import javax.swing.ImageIcon;

/**
 * The ghosts are Pac-Man's infinite enemies.
 * @author Martin Pieck
 */
public class Ghost extends Sprite implements Runnable {

    public Ghost(int x, int y, Level levelt, String difficulty) {
        level = levelt;
        coordinate[0] = x;
        coordinate[1] = y;
        iconCount = 2;
        iconNr = 1;
        animationTotalTime = animationCycleTime = 20;
        ingame = true;
        scared = false;
        pause = false;
        blinking = false;
        if (difficulty.contains("normal")) { randomfactor = 30; } else { randomfactor = 80; }
    }

    public void setPause(boolean pause){
        this.pause = pause;
    }    

    @Override
    public void cycleState() {
        // do nothing, no states
    }
    
    public void setImage() {
        Integer scaredflag = (scared) ? 1 : 0;
        if (scared && blinking) { scaredflag = 2; }
        String filename = "/img/" + this.getClass().getSimpleName().toLowerCase() + scaredflag.toString()  + iconNr.toString() + ".gif";
        ImageIcon ii = new ImageIcon(this.getClass().getResource(filename));
        image = ii.getImage();
    }

    @Override
    public void run() {
        long beforeTime, timeDiff, sleepTime, excessTime;
        excessTime = 0;
        beforeTime = System.currentTimeMillis();
        while (ingame) {
            while (pause) {
                try {
                    Thread.sleep(100);
                    beforeTime = System.currentTimeMillis();
                } catch (InterruptedException ex) {
                    Logger.getLogger(Game.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
                }
            }
            if (scared) {
                ghostScaredAlgorithm();
            } else { 
                ghostAlgorithm();
            }            
            timeDiff = System.currentTimeMillis() - beforeTime;
            sleepTime = Global.FRAMETIME.value() - timeDiff - excessTime;
            if (sleepTime >= 0) {
                try {
                    Thread.sleep(sleepTime);
                } catch (InterruptedException e) {
                    System.out.println("interrupted");
                }
            } else {
                excessTime -= sleepTime;
            }

            beforeTime = System.currentTimeMillis();

            while ((excessTime > Global.FRAMETIME.value())) {
                excessTime -= Global.FRAMETIME.value();
                if (scared) {
                    ghostScaredAlgorithm();
                } else { 
                    ghostAlgorithm();
                }  
            }
        }
    }

    public String randomGhostWalk() {
        String desireddirection = "up";
        int dir = (int) Math.ceil(Math.random() * 4);
        switch (dir) {
            case 1:
                desireddirection = "up";
                break;
            case 2:
                desireddirection = "down";
                break;
            case 3:
                desireddirection = "left";
                break;
            case 4:
                desireddirection = "right";
                break;
        }
        return desireddirection;


    }
    
    public void ghostAlgorithm() {
        int[] paccoords = { (int) Math.floor(level.pacman.coordinate[0] / Global.BLOCKSIZE.value()) , (int) Math.floor(level.pacman.coordinate[1] / Global.BLOCKSIZE.value()) };
        int[] ghostcoords = { (int) Math.floor((coordinate[0]) / Global.BLOCKSIZE.value()), (int) Math.floor((coordinate[1]) / Global.BLOCKSIZE.value()) };
        double distance = 40;
        String desireddirection;
        desireddirection = "up";
        
        if(level.levelArray[ghostcoords[1]-1][ghostcoords[0]].getEffect(this)) {
            double temp = Math.sqrt(Math.pow(Math.abs(paccoords[0]-ghostcoords[0]), 2) + Math.pow(Math.abs(paccoords[1]-(ghostcoords[1]-1)), 2));
            if (temp < distance && direction != Direction.DOWN) {
                distance = temp;
                desireddirection = "up";
            }
        }
        if(level.levelArray[ghostcoords[1]+1][ghostcoords[0]].getEffect(this)) {
            double temp = Math.sqrt(Math.pow(Math.abs(paccoords[0]-ghostcoords[0]), 2) + Math.pow(Math.abs(paccoords[1]-(ghostcoords[1]+1)), 2));
            if (temp < distance && direction != Direction.UP) {
                distance = temp;
                desireddirection = "down";
            }
        }
        if(level.levelArray[ghostcoords[1]][ghostcoords[0]-1].getEffect(this)) {
            double temp = Math.sqrt(Math.pow(Math.abs(paccoords[0]-(ghostcoords[0]-1)), 2) + Math.pow(Math.abs(paccoords[1]-ghostcoords[1]), 2));
            if (temp < distance && direction != Direction.RIGHT) {
                distance = temp;
                desireddirection = "left";
            }
        }
        if(level.levelArray[ghostcoords[1]][ghostcoords[0]+1].getEffect(this)) {
            double temp = Math.sqrt(Math.pow(Math.abs(paccoords[0]-(ghostcoords[0]+1)), 2) + Math.pow(Math.abs(paccoords[1]-ghostcoords[1]), 2));
            if (temp < distance && direction != Direction.LEFT) {
                distance = temp;
                desireddirection = "right";
            }
        }
        int dir = (int) Math.ceil(Math.random() * 100);
        if (dir > randomfactor) {
            desireddirection = randomGhostWalk();
        }
        changeDirection(desireddirection);
    }
    
    public void ghostScaredAlgorithm() {
        int[] paccoords = { (int) Math.floor(level.pacman.coordinate[0] / Global.BLOCKSIZE.value()) , (int) Math.floor(level.pacman.coordinate[1] / Global.BLOCKSIZE.value()) };
        int[] ghostcoords = { (int) Math.floor((coordinate[0]) / Global.BLOCKSIZE.value()), (int) Math.floor((coordinate[1]) / Global.BLOCKSIZE.value()) };
        double distance = 40;
        String desireddirection;
        desireddirection = "up";
        
        if(level.levelArray[ghostcoords[1]-1][ghostcoords[0]].getEffect(this)) {
            double temp = Math.sqrt(Math.pow(Math.abs(paccoords[0]-ghostcoords[0]), 2) + Math.pow(Math.abs(paccoords[1]-(ghostcoords[1]-1)), 2));
            if (temp > distance && direction != Direction.DOWN) {
                distance = temp;
                desireddirection = "up";
            }
        }
        if(level.levelArray[ghostcoords[1]+1][ghostcoords[0]].getEffect(this)) {
            double temp = Math.sqrt(Math.pow(Math.abs(paccoords[0]-ghostcoords[0]), 2) + Math.pow(Math.abs(paccoords[1]-(ghostcoords[1]+1)), 2));
            if (temp > distance && direction != Direction.UP) {
                distance = temp;
                desireddirection = "down";
            }
        }
        if(level.levelArray[ghostcoords[1]][ghostcoords[0]-1].getEffect(this)) {
            double temp = Math.sqrt(Math.pow(Math.abs(paccoords[0]-(ghostcoords[0]-1)), 2) + Math.pow(Math.abs(paccoords[1]-ghostcoords[1]), 2));
            if (temp > distance && direction != Direction.RIGHT) {
                distance = temp;
                desireddirection = "left";
            }
        }
        if(level.levelArray[ghostcoords[1]][ghostcoords[0]+1].getEffect(this)) {
            double temp = Math.sqrt(Math.pow(Math.abs(paccoords[0]-(ghostcoords[0]+1)), 2) + Math.pow(Math.abs(paccoords[1]-ghostcoords[1]), 2));
            if (temp > distance && direction != Direction.LEFT) {
                distance = temp;
                desireddirection = "right";
            }
        }
        int dir = (int) Math.ceil(Math.random() * 100);
        if (dir > 70) {
            desireddirection = randomGhostWalk();
        }
        changeDirection(desireddirection);
    }    
    
    public void reverse() {
        if (direction == Direction.LEFT) { direction = Direction.RIGHT;}
        else if(direction == Direction.RIGHT) { direction = Direction.LEFT; }
        else if(direction == Direction.UP) { direction = Direction.DOWN; }
        else if(direction == Direction.DOWN) { direction = Direction.UP; }
    }

    private boolean ingame;
    public boolean scared;
    private boolean pause;
    private int randomfactor;
}
