/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package game;

import java.io.*;
import java.util.Scanner;
import block.*;
import java.util.Iterator;
import leveleditor.BlockBuilder;
import main.Global;
import main.SoundEffects;
import score.Score;

/**
 *
 * @author s070280
 */
public class Level {

    /**
     * Creates a new instance of Blockbuilder wich creates a hashmap of all blocks.
     */
    public Level(String difficulti, Score score) {
        blBuilder = new BlockBuilder();
        lives = 3;
        lvlScore = score;
        difficulty = difficulti;
    }

    /**
     * Creates a gamefield by getting the stored level file, setting width and height.
     * It then creates and fills the array blocks[][] with blocks as described by the values taken from the level file.
     * [TODO] It then creates a score and calls score.initPoints().
     * (The array blocks[][] will be width+1 by height+1 elements, the elements at the edge will be filled with static walls.)
     * @param levelName (for example "test.level")
     */
    public void initLevel(String levelName) {
        /* copying a file using classes Scanner, PrintWriter, and File */
        Scanner sc;
        File sourceFile;
        String[] line;
        try {
            sourceFile = new File("levels/" + levelName + ".level");
            sc = new Scanner(sourceFile);    
            
            line = sc.nextLine().split(":");
            width = Integer.valueOf(line[0]);
            height = Integer.valueOf(line[1]);
            
            levelArray = new Block[height][width];                       
            for (int i = 0; i < height; i++) {               
                line = sc.nextLine().split(":");
                for (int j = 0; j < width; j++) {                    
                    levelArray[i][j] = blBuilder.getBlockfromString(line[j]); 
                }
            }      
        } catch (FileNotFoundException e) {
            System.out.println("Could not open file due to");
            System.out.println(e);
        }      
    }

    /**
     * Saves the array blocks[][] by writing a level file containing the level configuration with the name of the levelName parameter.
     * @param levelName 
     */
    public void save(String levelName) {
        /* copying a file using classes Scanner, PrintWriter, and File */
        PrintWriter pw;
        File targetFile;

        try {

            targetFile = new File("levels/" + levelName + ".level");
            pw = new PrintWriter(targetFile);
            width = levelArray[0].length;
            height = levelArray.length;
            
            pw.println(width + ":" + height);

            for (int i = 0; i < height; i++) {
                for (int j = 0; j < width; j++) {
                    pw.print(levelArray[i][j].getSaveTypeBlock());  
                    if (j < (width-1)){
                        pw.print(":");
                    }
                }
                pw.println();
            }
            pw.close();
            
        } catch(IOException e) {
            System.err.format("createFile error: %s%n", e);
        }
    }

    /**
     * Creates an empty gamefield by creating an array blocks[][] of width+1 by height+1 elements,
     * fills the elements at the edge with static walls, and all remaining elements with empty paths.
     * @param width
     * @param height 
     * 
     */
    public void createEmpty(int deswidth, int desheight) {
        width = deswidth;
        height = desheight;

        levelArray = new Block[height][width];

        for (int i = 0; i < height; i++) {
            for (int j = 0; j < width; j++) {
                if (i == 0 || j == 0 || i == height - 1 || j == width - 1) // zorgt voor de rand walls
                {
                    levelArray[i][j] = new StaticWall();
                } else {
                    levelArray[i][j] = new Path("empty");
                }

            }
        }

    }

    public Block[][] getLevel() {
        return levelArray;
    }

    public void createSprites() {
        int lengtharray = countBlocks("spawn_ghost");
        ghostSpawns = new Spawn[lengtharray];
        ghosts = new Ghost[lengtharray];
        ghostThreads = new Thread[lengtharray];
        
        int ghostcounter = 0;
        
        for (int y = 0; y < levelArray.length; y++) {
            for (int x = 0; x < levelArray[0].length; x++) {
                if (levelArray[y][x].getSaveTypeBlock().equals("spawn_pacman")) {
                    pacSpawn = (Spawn) levelArray[y][x];
                    pacSpawn.spawnSprite(this, x*30, y*30);
                } else if (levelArray[y][x].getSaveTypeBlock().equals("spawn_ghost")) {
                    ghostSpawns[ghostcounter] = (Spawn) levelArray[y][x];
                    ghostSpawns[ghostcounter].spawnSprite(this, x*30, y*30, ghostcounter);
                    ghostcounter++;
                }
            }
        }
    }
    
    public void setWarpHoles(){
        int i = 0;
        boolean warp = false;
        int[][] warps = new int[2][2];        
        for(int y=0;y<levelArray.length;y++){
            for(int x=0;x<levelArray[0].length;x++){
                if(levelArray[y][x].getClass().getSimpleName().toLowerCase().equals("warphole")){                    
                    warp = true;
                    warps[i][0] = y;
                    warps[i][1] = x;
                    i++;
                }
            }
        }   
        if(warp){
            WarpHole warp1 = (WarpHole) levelArray[warps[0][0]][warps[0][1]];
            WarpHole warp2 = (WarpHole) levelArray[warps[1][0]][warps[1][1]];
            warp1.setNextWarpHole(warps[1]);
            warp2.setNextWarpHole(warps[0]);
        }
    }
    
    public void setMovingWalls(){
        int lengtharray = countBlocks("movingwall");
        movingWallsTotal = new MovingWall[lengtharray];
        int i = 0;
        for(int y=0;y<levelArray.length;y++){
            for(int x=0;x<levelArray[0].length;x++){
                if(levelArray[y][x].getSaveTypeBlock().equals("movingwall")){    
                    movingWallsTotal[i] = (MovingWall)levelArray[y][x];
                    i++;
                }
            }
        }                       
    }
    
    /**
     * This functions counts how many blocks of a specific type are present in the level.
     * Needed for checks like max 2 warpholes per level and max 1 pacspawn per level.
     * @return 
     */
    public int countBlocks(String blocktype) {
        int count = 0;
        for(int y=0;y<levelArray.length;y++){
            for(int x=0;x<levelArray[0].length;x++){
                if(levelArray[y][x].getSaveTypeBlock().equals(blocktype)){                    
                    count++;
                }
            }
        } 
        return count;
    }
    
    public void setGhostsScared(boolean testvalue) {
        
        for(int i = 0; i < ghosts.length; i++) {
            ghosts[i].scared = testvalue;
            ghosts[i].reverse();
        }
        
    }
    
    public void setGhostsBlinking(boolean testvalue) {

        for(int i = 0; i < ghosts.length; i++) {
            ghosts[i].blinking = testvalue;
            ghosts[i].setImage();
        }
        
    }
    
    /**
     * Checks whether or not the coordinates of Pac-Man and any ghost are of such values that the sprites collide
     * If they do, attribute scared of Ghost will be checked.
     * If the ghost is scared, points will be added, and the ghost will be deleted. A new ghost will be spawned at an appropriate spawn.
     * If the ghost is not scared, attribute lives will be decreased by one, PacMan will be deleted, and recreated at the appropriate spawn.
     * This method will then return the current number of lives.
     * @return 
     */
    public void checkBite() {
        
        double longestdistance = 0;
        int[] spawncoords = new int[2];
        
        for(int k = 0; k < ghosts.length; k++) {
            if (Math.abs(ghosts[k].coordinate[0] - pacman.coordinate[0]) < 30 && Math.abs(ghosts[k].coordinate[1] - pacman.coordinate[1]) < 30 ) {
                if (ghosts[0].scared == true) {
                    
                    Thread tsound = new Thread(new SoundEffects("eat"));
                    tsound.start();
                    
                    lvlScore.addPoints(2000);
                    
                    //pacman is in supermode, ghost needs to respawn at far most ghostspawn
                    //search for far most ghostspawn
                    for (int i = 0; i < height; i++) {

                        for (int j = 0; j < width; j++) {

                            if(levelArray[i][j].getSaveTypeBlock().equals("spawn_ghost")){    
                                //calculate distace from the spawn to pacman
                                double distance = Math.sqrt(Math.pow(Math.abs(pacman.coordinate[0]-j), 2) + Math.pow(Math.abs(pacman.coordinate[1]-i), 2));
                                if (distance > longestdistance) {
                                    longestdistance = distance;
                                    spawncoords[0] = j;
                                    spawncoords[1] = i;
                                }
                            }

                        }
                    }
                    ghosts[k].coordinate[0] = spawncoords[0]*30;
                    ghosts[k].coordinate[1] = spawncoords[1]*30;
                    //ghosts[k].scared = false;
                } else {
                    Thread tsound = new Thread(new SoundEffects("death"));
                    tsound.start();
                    
                    pacSpawn.respawnSprite(); 
                    lives--;
                }             
            }
        }
        
    }

    /**
     * Checks if all the pac-dots are eaten by scanning through the blocks[][] array and testing if
     * there is a block left of type path.PacDot, if so it returns false, else it returns true.
     * @return
     */
    public boolean checkAllPacDotsEaten() {
        boolean allPacDotsEaten = true;
        for (int i = 0; i < height; i++) {

            for (int j = 0; j < width; j++) {

                if ( levelArray[i][j].getSaveTypeBlock().equals("path_pacdot") ) {
                    allPacDotsEaten = false;
                    return allPacDotsEaten;
                }

            }
        }        
        return allPacDotsEaten;
    }    
    
    public int[] getCoordsPacMan() {
        return pacman.coordinate;
    }
    
    public int width;          //This attribute describes the number of horizontal blocks of which the game field consists.
    public int height;         //This attribute describes the number of vertical blocks of which the game field consists.
    public Block[][] levelArray;    //This attribute is an array of objects of type Block. It contains all the blocks at their respective coordinates in the game field.
    public PacMan pacman;       //This attribute represents the Pac-Man sprite in the level. 
    public Spawn pacSpawn;
    public MovingWall[] movingWallsTotal;
    public int lives;           //This attribute describes the amount of lives the player has left in the current level.
    private BlockBuilder blBuilder;  
    private Score lvlScore;
    public Ghost[] ghosts; //This attribute is an array of objects of type sprite that describe the position of the ghosts on a block.
    public Thread[] ghostThreads;
    public Spawn[] ghostSpawns;
    public String difficulty;
}