
package score;

import java.io.*;
import java.util.Scanner;


public class Score {
    
    // Creates a score with 
    // current score = setScore
    // and points = 0
    public Score() {
        currentScore = 0;
        points = 0;
    }
    
    // returns the current number of points.
    public int getPoints() {
        return points;
    }
    
    // Retrieves the top 5 best scores for level levelName and returns
    //  them in an array of integers.
    // where levelName = "level" will return the top 5 scores
    //  for the default levels.
    public int[] getTopScores(String levelName) {
        int[] list = new int[5];
        
        try {
            sourceFile = new File("scores/"+levelName+".score");
            sc = new Scanner(sourceFile);
                
            for(int i=0 ; i<5 ; i++) {
                if(sc.hasNextInt()) {list[i]= sc.nextInt();}
            }
            
        } catch (FileNotFoundException ex) {
            System.out.println("Could not open file due to");
            System.out.println(ex);
        }
        
        return list;
    }
    
    // Checks whether or not currentScore is higher than any of
    //  the 5 topscores for level levelName (where levelName = "level" 
    //  will be interpreted as the default levels)
    // Return: The position of the inserted score (0...4) or -1 if no new
    //  score has been inserted.
    public int checkTopscore(String levelName) {
        
        int[] scores = getTopScores(levelName);
        
        for(int i=0 ; i<=4 ; i++) {
            if(currentScore>scores[i]) {
                return i;
            }
        }
        return -1;
        
    }
    
    // Places currentScore and playerName at position p in the highscores for 
    //  levelName and moves the other scores down accordingly, overwriting the 
    //  lowest score.
    public void saveTopscore(String levelName, String playerName) {
        if(!playerName.equals("Player") && !playerName.equals("")) {
            int[] scores = getTopScores(levelName);  
            String[] names = getTopNames(levelName);
            int p = checkTopscore(levelName);

            // Move old scores and names
            for(int k=4 ; k>p ; k--) {
                scores[k]=scores[k-1]; 
                names[k]=names[k-1];
            }
            // Save new score and name
            scores[p]=currentScore;
            names[p]=playerName;
            //

            writeScores(levelName,scores,names);  // Overwrite the saved scores
        }     
    }
    
    // Retrieves the names for the top 5 best scores for level levelName and 
    //  returns them in an array of strings
    // where levelName = "level" will return the top 5 scores
    //  for the default levels.    
    public String[] getTopNames(String levelName) {
        String[] names = new String[5];
        
        try {
            sourceFile = new File("scores/"+levelName+".score");
            sc = new Scanner(sourceFile);
                
            String line = sc.nextLine();
            
            for(int i=0 ; i<5 ; i++) {
                if(sc.hasNextLine()) {
                    line = sc.nextLine();
                    names[i] = line;
                }
            }
            
        } catch (FileNotFoundException ex) {
            System.out.println("Could not open file due to");
            System.out.println(ex);
        }
        
        
        return names;
    }
    
    
    // Writes the topscores and topnames for levelName to file.
    private void writeScores(String levelName, int[] scores, String[] names) {
        try {
            targetFile = new File("scores/"+levelName+".score");
            pw = new PrintWriter(targetFile);
            
            for(int i=0 ; i<5 ; i++) {
                if(scores[i]!=0) {
                    pw.print(scores[i]+" ");
                }
            }
            
            for(int j=0 ; j<5 ; j++) {
                if(names[j]!=null){
                    pw.println();
                    pw.print(names[j]);
                }
            }
            pw.close();
            
        } catch (FileNotFoundException ex) {
            System.out.println("Could not save file due to");
            System.out.println(ex);
        }
        
    }
    
    public void addPoints(int addPoints) {
        points = points + addPoints;
    }
    
    public void setScore(int addPoints, long timeFactor) {
        long bonusPoints = 0;
        if(timeFactor < 1) {
            bonusPoints = 20000;
        }
        else {
            bonusPoints = addPoints / timeFactor;
        }
        currentScore = currentScore + points + (int) bonusPoints;
        points = 0;
    }
    
    public int currentScore;
    private int points;
    private Scanner sc;
    private File sourceFile;
    private File targetFile;
    private PrintWriter pw;
}
